# This cmake file configures installing Qul and supporting platform libraries

macro(qul_private_install_lib_with_symlinks library_path)
    unset(lib_with_symlinks)
    if(UNIX)
        file(GLOB lib_with_symlinks "${library_path}*[!.debug]")
    endif()
    install(
        FILES
            ${library_path}
            ${lib_with_symlinks}
        DESTINATION ${_ARGS_DESTINATION}
        COMPONENT ${_ARGS_COMPONENT}
        CONFIGURATIONS ${_ARGS_CONFIGURATIONS}
        ${_ARGS_UNPARSED_ARGUMENTS}
    )
endmacro()

function(qul_private_install_shared_libs)
    set(options)
    set(oneValues DESTINATION COMPONENT SOURCE_DIR)
    set(multipleValues LIBS_NAMES FILES CONFIGURATIONS)
    cmake_parse_arguments(_ARGS "${options}" "${oneValues}" "${multipleValues}" ${ARGN})
    foreach(lib_name ${_ARGS_LIBS_NAMES})
        if(NOT MINGW)
            set(LIB_PREFIX ${CMAKE_SHARED_LIBRARY_PREFIX})
        endif()
        set(lib_path ${_ARGS_SOURCE_DIR}/${LIB_PREFIX}${lib_name}${CMAKE_SHARED_LIBRARY_SUFFIX})
        qul_private_install_lib_with_symlinks(${lib_path})
    endforeach()
    foreach(lib_path ${_ARGS_FILES})
        qul_private_install_lib_with_symlinks(${lib_path})
    endforeach()
endfunction()

function(qul_private_install_static_libs)
    set(options)
    set(oneValues DESTINATION COMPONENT SOURCE_DIR)
    set(multipleValues LIBS_NAMES FILES CONFIGURATIONS)
    cmake_parse_arguments(_ARGS "${options}" "${oneValues}" "${multipleValues}" ${ARGN})
    foreach(lib_name ${_ARGS_LIBS_NAMES})
        set(lib_path ${_ARGS_SOURCE_DIR}/${CMAKE_STATIC_LIBRARY_PREFIX}${lib_name}${CMAKE_STATIC_LIBRARY_SUFFIX})
        qul_private_install_lib_with_symlinks(${lib_path})
    endforeach()
    foreach(lib_path ${_ARGS_FILES})
        qul_private_install_lib_with_symlinks(${lib_path})
    endforeach()
endfunction()

include(${CMAKE_CURRENT_LIST_DIR}/install_monotype.cmake OPTIONAL)

file(MAKE_DIRECTORY ${Qul_BINARY_DIR}/include/qul)

install(DIRECTORY ${PROJECT_SOURCE_DIR}/include/qul
        DESTINATION "${CMAKE_INSTALL_INCLUDEDIR}"
        COMPONENT CoreHeaders)

install(DIRECTORY ${CMAKE_BINARY_DIR}/include/qul DESTINATION "${CMAKE_INSTALL_INCLUDEDIR}" COMPONENT CoreHeaders)
set(CMAKE_INSTALL_PLUGINSDIR plugins)

include(GNUInstallDirs)

# The main export is defined in src/CMakeLists.txt right after
# add_library(Qul)

export(PACKAGE Qul)

if(QUL_BUILD_FRAMEWORK)
    if(NOT QUL_CORE_TARGETS_BASE_NAME)
        message(FATAL_ERROR "Internal error: QUL_CORE_TARGETS_BASE_NAME is not set")
    endif()

    install(TARGETS Core PNGDecoderLodePNG PNGDecoderNull
            EXPORT ${QUL_CORE_TARGETS_BASE_NAME}
            LIBRARY DESTINATION ${CMAKE_INSTALL_LIBDIR} COMPONENT CoreLibrary
            ARCHIVE DESTINATION ${CMAKE_INSTALL_LIBDIR} COMPONENT CoreLibrary
            PUBLIC_HEADER DESTINATION "${CMAKE_INSTALL_INCLUDEDIR}/qul" COMPONENT CoreHeaders
           )

    export(TARGETS PlatformArchitecture APPEND
           NAMESPACE Qul::
           FILE "${PROJECT_BINARY_DIR}/${QUL_CORE_TARGETS_BASE_NAME}.cmake"
            )

    install(TARGETS PlatformArchitecture
            EXPORT ${QUL_CORE_TARGETS_BASE_NAME}
            DESTINATION ${CMAKE_INSTALL_LIBDIR} COMPONENT CoreLibrary)
endif()

if(QUL_BUILD_PLATFORM)
    if(NOT QUL_PLATFORM_TARGETS_BASE_NAME)
        message(FATAL_ERROR "Internal error: QUL_PLATFORM_TARGETS_BASE_NAME is not set")
    endif()

    install(FILES "${PROJECT_BINARY_DIR}/QulPlatformTargets_${QUL_PLATFORM}_extra_config.cmake"
            DESTINATION "${INSTALL_CMAKE_DIR}/QulPlatformTargets"
            COMPONENT PlatformSources OPTIONAL)
endif()

install(DIRECTORY "${PROJECT_SOURCE_DIR}/src/3rdparty/qoi"
        DESTINATION "src/3rdparty"
        COMPONENT PlatformBase)

install(DIRECTORY "${PROJECT_SOURCE_DIR}/src/3rdparty/minihdlc"
        DESTINATION "src/3rdparty"
        COMPONENT PlatformBase)

install(FILES "${PROJECT_SOURCE_DIR}/src/3rdparty/NanopbConfig.cmake"
        DESTINATION "src/3rdparty"
        COMPONENT PlatformBase)

install(DIRECTORY "${PROJECT_SOURCE_DIR}/src/3rdparty/nanopb"
        DESTINATION "src/3rdparty"
        COMPONENT PlatformBase
        PATTERN "3rdparty/nanopb/spm_headers" EXCLUDE # not needed, and does include symlinks
        PATTERN "3rdparty/nanopb/generator/protoc" EXCLUDE)

install(FILES "${PROJECT_SOURCE_DIR}/src/3rdparty/nanopb/generator/protoc"
        DESTINATION "src/3rdparty/nanopb/generator"
        PERMISSIONS OWNER_EXECUTE GROUP_EXECUTE WORLD_EXECUTE OWNER_READ GROUP_READ WORLD_READ OWNER_WRITE
        COMPONENT PlatformBase)

install(DIRECTORY "${PROJECT_SOURCE_DIR}/platform/compatibility"
        DESTINATION "platform"
        COMPONENT PlatformBase)

if(NOT CMAKE_CROSSCOMPILING)
    # Binaries
    find_package(Qt6 6.2 COMPONENTS Core Gui Xml LinguistTools REQUIRED)

    # Look for shared libreries for clang and freetype, but search only in CMAKE specific directories,
    # as looking in system PATH can result in installation of wrong dlls from unrelated software
    if (WIN32)
        find_file(FREETYPE_SHARED_LIB_PATH freetype.dll NO_SYSTEM_ENVIRONMENT_PATH PATH_SUFFIXES lib)
        if (NOT FREETYPE_SHARED_LIB_PATH)
            message(FATAL_ERROR "Could not find freetype.dll for installation")
        endif()
        find_file(LIBCLANG_SHARED_LIB_PATH libclang.dll NO_SYSTEM_ENVIRONMENT_PATH PATH_SUFFIXES bin)
    else (UNIX)
        find_library(LIBLLVM_SHARED_LIB_PATH libLLVM-13.so libLLVM.so libLLVM.so.13 PATHS ${CLANG_RESOURCE_DIR}/../.. NO_SYSTEM_ENVIRONMENT_PATH)
        if (NOT LIBLLVM_SHARED_LIB_PATH)
            message(FATAL_ERROR "Could not find libLLVM for installation")
        endif()
        find_library(LIBCLANG_SHARED_LIB_PATH libclang-13.so libclang.so libclang.so.13 PATHS ${CLANG_RESOURCE_DIR}/../.. NO_SYSTEM_ENVIRONMENT_PATH)
        find_library(LIBFFI_SHARED_LIB_PATH libffi.so PATHS ${CLANG_RESOURCE_DIR}/../.. NO_SYSTEM_ENVIRONMENT_PATH)
    endif()
    if (NOT LIBCLANG_SHARED_LIB_PATH)
        message(FATAL_ERROR "Could not find libclang for installation")
    endif()

    install(
        PROGRAMS
            "${PACKAGE_PREFIX_DIR}/bin/lrelease${CMAKE_EXECUTABLE_SUFFIX}"
            "${PACKAGE_PREFIX_DIR}/bin/lupdate${CMAKE_EXECUTABLE_SUFFIX}"
            "${PACKAGE_PREFIX_DIR}/bin/lconvert${CMAKE_EXECUTABLE_SUFFIX}"
        DESTINATION ${CMAKE_INSTALL_BINDIR}
        COMPONENT Tools
    )

    set(QUL_STATIC_LIBRARIES_SOURCE_DIR ${PACKAGE_PREFIX_DIR}/lib)
    set(QUL_STATIC_LIBRARIES_DEST_DIR ${CMAKE_INSTALL_LIBDIR})
    if (WIN32)
        set(QUL_SHARED_LIBRARIES_SOURCE_DIR ${PACKAGE_PREFIX_DIR}/bin)
        set(QUL_SHARED_LIBRARIES_DEST_DIR ${CMAKE_INSTALL_BINDIR})
        set(QUL_SHARED_LIBRARIES_BY_TOOLCHAIN_DEST_DIR ${CMAKE_INSTALL_LIBDIR}/${QUL_COMPILER_NAME})
    else (UNIX)
        set(QUL_SHARED_LIBRARIES_SOURCE_DIR ${PACKAGE_PREFIX_DIR}/lib)
        set(QUL_SHARED_LIBRARIES_DEST_DIR ${CMAKE_INSTALL_LIBDIR})
    endif()

    qul_private_install_shared_libs(
        SOURCE_DIR
            ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
        FILES
            ${FREETYPE_SHARED_LIB_PATH}
            ${LIBCLANG_SHARED_LIB_PATH}
            ${LIBLLVM_SHARED_LIB_PATH}
        DESTINATION ${QUL_SHARED_LIBRARIES_DEST_DIR}
        COMPONENT Tools
    )
    if (NOT QUL_PRIVATE_HOST_TOOLS_WITH_STATIC_QT)
        qul_private_install_shared_libs(
            SOURCE_DIR
                ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
            LIBS_NAMES
                Qt6Core
                Qt6Gui
                Qt6Widgets
                Qt6Network
                Qt6Xml
                Qt6Svg
                Qt6Qml
            DESTINATION ${QUL_SHARED_LIBRARIES_DEST_DIR}
            COMPONENT Tools
        )

        # Image plugins are required for all platforms (used by qulrcc)
        install(
            DIRECTORY "${PACKAGE_PREFIX_DIR}/plugins/imageformats"
            DESTINATION ${CMAKE_INSTALL_PLUGINSDIR}
            COMPONENT Tools
            FILES_MATCHING
            PATTERN "*${CMAKE_SHARED_LIBRARY_SUFFIX}"
            PATTERN  "*d.dll" EXCLUDE
            PATTERN "*.so.debug" EXCLUDE
        )
    endif()

    qul_private_install_shared_libs(
        SOURCE_DIR
            ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
        LIBS_NAMES
            Qt6SerialPort
            Qt6WebSockets
        DESTINATION ${QUL_SHARED_LIBRARIES_DEST_DIR}
        COMPONENT HardwareCloud
    )

    if (WIN32)
        # Release libs
        qul_private_install_shared_libs(
            SOURCE_DIR
                ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
            LIBS_NAMES
                Qt6Core
                Qt6Gui
                Qt6Widgets
                Qt6Qml
                Qt6Test
            DESTINATION ${QUL_SHARED_LIBRARIES_BY_TOOLCHAIN_DEST_DIR}
            COMPONENT CoreLibrary
        )
        qul_private_install_shared_libs(
            SOURCE_DIR
                ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
            LIBS_NAMES
                Qt6SerialPort
                Qt6WebSockets
            DESTINATION ${QUL_SHARED_LIBRARIES_DEST_DIR}
            COMPONENT HardwareCloud
        )

        qul_private_install_static_libs(
            SOURCE_DIR
                ${QUL_STATIC_LIBRARIES_SOURCE_DIR}
            LIBS_NAMES
                Qt6Core
                Qt6Widgets
                Qt6Gui
                Qt6Qml
                Qt6Test
            DESTINATION ${QUL_STATIC_LIBRARIES_DEST_DIR}
            COMPONENT CoreLibrary
        )

        qul_private_install_shared_libs(
            SOURCE_DIR
                ${PACKAGE_PREFIX_DIR}/plugins/platforms/
            LIBS_NAMES
                qwindows
            DESTINATION ${QUL_SHARED_LIBRARIES_BY_TOOLCHAIN_DEST_DIR}
            COMPONENT CoreLibrary
        )
        if (MSVC)
            # Debug libs are needed only for MSVC builds
            qul_private_install_shared_libs(
                SOURCE_DIR
                    ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
                LIBS_NAMES
                    Qt6Cored
                    Qt6Guid
                    Qt6Widgetsd
                    Qt6Qmld
                    Qt6Testd
                DESTINATION ${QUL_SHARED_LIBRARIES_BY_TOOLCHAIN_DEST_DIR}
                COMPONENT CoreLibrary
                CONFIGURATIONS Debug
            )

            qul_private_install_static_libs(
                SOURCE_DIR
                    ${QUL_STATIC_LIBRARIES_SOURCE_DIR}
                LIBS_NAMES
                    Qt6Core
                    Qt6Widgets
                    Qt6Gui
                    Qt6Qml
                    Qt6Test
                DESTINATION ${QUL_STATIC_LIBRARIES_DEST_DIR}
                COMPONENT CoreLibrary
                CONFIGURATIONS Debug
            )

            qul_private_install_shared_libs(
                SOURCE_DIR
                    ${PACKAGE_PREFIX_DIR}/plugins/platforms/
                LIBS_NAMES
                    qwindowsd
                DESTINATION ${QUL_SHARED_LIBRARIES_BY_TOOLCHAIN_DEST_DIR}
                COMPONENT CoreLibrary
                CONFIGURATIONS Debug
            )
        endif()
    elseif(UNIX AND NOT APPLE)
        find_library(LIBFFI_SHARED_LIB_PATH libffi.so NO_SYSTEM_ENVIRONMENT_PATH)

        qul_private_install_shared_libs(
            SOURCE_DIR
                ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
            LIBS_NAMES
                Qt6Core
                Qt6Gui
                Qt6Widgets
                Qt6Qml
                Qt6Test
            DESTINATION ${QUL_SHARED_LIBRARIES_DEST_DIR}
            COMPONENT CoreLibrary
        )
        qul_private_install_shared_libs(
            SOURCE_DIR
                ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
            LIBS_NAMES
                Qt6Network
                Qt6Xml
                Qt6Svg
                Qt6DBus
                Qt6XcbQpa
                Qt6OpenGL
                icui18n
                icuuc
                icudata
            FILES
                ${LIBFFI_SHARED_LIB_PATH}
            DESTINATION ${QUL_SHARED_LIBRARIES_DEST_DIR}
            COMPONENT CoreLibrary
        )
        qul_private_install_shared_libs(
            SOURCE_DIR
                ${QUL_SHARED_LIBRARIES_SOURCE_DIR}
            LIBS_NAMES
                Qt6SerialPort
                Qt6WebSockets
            DESTINATION ${QUL_SHARED_LIBRARIES_DEST_DIR}
            COMPONENT HardwareCloud
        )

        qul_private_install_shared_libs(
            SOURCE_DIR
                ${PACKAGE_PREFIX_DIR}/plugins/platforms/
            LIBS_NAMES
                qxcb
            DESTINATION ${CMAKE_INSTALL_PLUGINSDIR}/platforms
            COMPONENT CoreLibrary
        )

        install(
            FILES "${CMAKE_CURRENT_SOURCE_DIR}/util/qt.conf"
            DESTINATION ${CMAKE_INSTALL_BINDIR}
            COMPONENT CoreLibrary
        )
    endif()
endif()

# CMake files
install(
    FILES "${Qul_CMAKE_DIR}/QulTranslationTools.cmake"
    DESTINATION "${INSTALL_CMAKE_DIR}"
    COMPONENT CoreBase
)

# CMake files
configure_file(${PROJECT_SOURCE_DIR}/src/QulConfig.cmake.in "${PROJECT_BINARY_DIR}/QulConfig.cmake" @ONLY)

set(CONF_APP_COMMON_DIR "\${Qul_CMAKE_DIR}/../../../src/app_common")
set(CONF_FONTS_DIR "\${Qul_CMAKE_DIR}/../../../src/3rdparty/fonts")
set(CONF_BIN_DIR "\${Qul_CMAKE_DIR}/../../../${CMAKE_INSTALL_BINDIR}")
set(CONF_LIB_DIR "\${Qul_CMAKE_DIR}/../../../${CMAKE_INSTALL_LIBDIR}")
set(CONF_CMAKE_MODULES_DIR "\${Qul_CMAKE_DIR}/extra-cmake-modules")

install(FILES "${Qul_CMAKE_DIR}/LegacyAssetsDefaults.cmake"
        DESTINATION "${INSTALL_CMAKE_DIR}"
        COMPONENT PlatformBase)

install(DIRECTORY "${CMAKE_SOURCE_DIR}/include/platform"
        DESTINATION "${CMAKE_INSTALL_INCLUDEDIR}"
        COMPONENT PlatformHeaders)

configure_file(${PROJECT_SOURCE_DIR}/src/QulConfig.cmake.in
               "${PROJECT_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/QulConfig.cmake"
               @ONLY)
configure_file(${PROJECT_SOURCE_DIR}/src/QulConfigVersion.cmake.in
               "${PROJECT_BINARY_DIR}/QulConfigVersion.cmake" @ONLY)


install(FILES "${PROJECT_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/QulConfig.cmake"
              "${PROJECT_BINARY_DIR}/QulConfigVersion.cmake"
              "${Qul_CMAKE_DIR}/QulScopeFinalizerHelpers.cmake"
              "${Qul_CMAKE_DIR}/QulMacros.cmake"
              "${Qul_CMAKE_DIR}/QulTargetPostProcess.cmake"
        DESTINATION "${INSTALL_CMAKE_DIR}"
        COMPONENT CoreBase)

install(FILES "${PROJECT_SOURCE_DIR}/CMakeLists.txt"
        DESTINATION "."
        COMPONENT CoreBase)

install(FILES "${Qul_CMAKE_DIR}/extra-cmake-modules/FindQt6Bundle.cmake"
        DESTINATION "${INSTALL_CMAKE_DIR}/extra-cmake-modules"
        COMPONENT CoreBase)

install(FILES "${Qul_CMAKE_DIR}/3rdparty/extra-cmake-modules/modules/ECMEnableSanitizers.cmake"
        DESTINATION "${INSTALL_CMAKE_DIR}/3rdparty/extra-cmake-modules/modules"
        COMPONENT CoreBase)

install(FILES "${PROJECT_SOURCE_DIR}/src/app_common/AppMacros.cmake"
        DESTINATION "src/app_common"
        COMPONENT CoreBase)

install(FILES "${Qul_CMAKE_DIR}/performancelogging.cmake"
        DESTINATION "${INSTALL_CMAKE_DIR}"
        COMPONENT CoreBase)

install(DIRECTORY "${PROJECT_SOURCE_DIR}/src/app_common/mains"
        DESTINATION "src/app_common"
        COMPONENT CoreBase)

install(DIRECTORY "${PROJECT_SOURCE_DIR}/src/app_common/freertos"
        DESTINATION "src/app_common"
        COMPONENT CoreBase)

install(DIRECTORY "${Qul_CMAKE_DIR}/toolchain"
        DESTINATION "${INSTALL_CMAKE_DIR}"
        COMPONENT CoreBase)

install(FILES "${PROJECT_SOURCE_DIR}/util/deviceinformationexchangeprotocol/deviceinformationexchangeprotocol.proto"
              "${PROJECT_SOURCE_DIR}/util/deviceinformationexchangeprotocol/deviceinformationexchangeprotocol.options"
        DESTINATION "util/deviceinformationexchangeprotocol"
        COMPONENT CoreBase)

install(DIRECTORY ${PROJECT_SOURCE_DIR}/platform/architecture
        DESTINATION "platform"
        COMPONENT PlatformBase)

if(QUL_BUILD_FRAMEWORK)
  install(EXPORT ${QUL_CORE_TARGETS_BASE_NAME} NAMESPACE Qul:: DESTINATION "${INSTALL_CMAKE_DIR}/QulCoreTargets" COMPONENT CoreLibrary)
endif()

# Examples
install(DIRECTORY ${PROJECT_SOURCE_DIR}/examples DESTINATION "." COMPONENT Examples
                                                                 PATTERN "*/test_automation" EXCLUDE
                                                                 PATTERN "*/examples/CMakeLists.txt" EXCLUDE)
install(DIRECTORY ${PROJECT_SOURCE_DIR}/demos DESTINATION "." COMPONENT Examples
                                              PATTERN "*/canbus/rh850-d1m1a-baremetal" EXCLUDE
                                              PATTERN "*/test_automation" EXCLUDE
                                              PATTERN "*/demos/CMakeLists.txt" EXCLUDE)
# Sources
install(DIRECTORY ${PROJECT_SOURCE_DIR}/src DESTINATION "." COMPONENT CoreSources
                                                            PATTERN "*/app_common" EXCLUDE
                                                            PATTERN "QulMonotypeIntegration.cmake" EXCLUDE # installed by QulMonotypeIntegration.cmake
                                                            PATTERN "*/monotype" EXCLUDE # installed by QulMonotypeIntegration.cmake
                                                            PATTERN "3rdparty/nanopb" EXCLUDE # already installed in PlatformSources
                                                            PATTERN "3rdparty/minihdlc" EXCLUDE # already installed in PlatformSources
                                                            )

install(FILES ${PROJECT_SOURCE_DIR}/src/install.cmake
              ${PROJECT_SOURCE_DIR}/src/QulConfig.cmake.in
              ${PROJECT_SOURCE_DIR}/src/QulConfigVersion.cmake.in
        DESTINATION "src" COMPONENT PlatformSources) # This is a workaround for UL-4168.
                                                     # These files should not be part of the platform sources.

if(EXISTS ${PROJECT_SOURCE_DIR}/tests)
    install(DIRECTORY ${PROJECT_SOURCE_DIR}/tests DESTINATION "." COMPONENT TestsSources)
endif()
if(EXISTS ${PROJECT_SOURCE_DIR}/test_automation)
    install(DIRECTORY ${PROJECT_SOURCE_DIR}/test_automation DESTINATION "." COMPONENT TestAutomation
                                                                            PATTERN "*/__pycache__" EXCLUDE
                                                                            )
endif()

install(DIRECTORY ${PROJECT_SOURCE_DIR}/include/platforminterface
        DESTINATION "${CMAKE_INSTALL_INCLUDEDIR}"
        COMPONENT CoreHeaders)

install(DIRECTORY ${Qul_SOURCE_DIR}/src/3rdparty/fonts
        DESTINATION "src/3rdparty"
        COMPONENT CoreBase)

# These files need to be optional because when an eval customer(which does not have CoreSources wants to install
# the eval version, these files are not available.
install(FILES "${PROJECT_SOURCE_DIR}/examples/CMakeLists.txt" DESTINATION "examples/" COMPONENT CoreSources OPTIONAL)
install(FILES "${PROJECT_SOURCE_DIR}/demos/CMakeLists.txt" DESTINATION "demos/" COMPONENT CoreSources OPTIONAL)
install(FILES "${PROJECT_SOURCE_DIR}/CMakeLists.txt" DESTINATION  "." COMPONENT CoreSources OPTIONAL)
install(DIRECTORY "${PROJECT_SOURCE_DIR}/platform/export" DESTINATION "platform/" COMPONENT PlatformBase OPTIONAL)

install(FILES ${PROJECT_SOURCE_DIR}/platform/CMakeLists.txt
        DESTINATION "platform"
        COMPONENT PlatformBase)
install(DIRECTORY "${CMAKE_SOURCE_DIR}/platform/common"
        DESTINATION "platform"
        COMPONENT PlatformBase)

# Package Generator  #######################################################

if(CMAKE_CROSSCOMPILING)
    set(CORE_PACKAGE_NAME "${QUL_COMPILER_NAME}-${QUL_PLATFORM_ARCHITECTURE}")
endif()
if(NOT CORE_PACKAGE_NAME AND (QUL_BUILD_PLATFORM OR QUL_BUILD_FRAMEWORK OR QUL_BUILD_TOOLS))
    message(FATAL_ERROR "Internal error: CORE_PACKAGE_NAME not set.")
endif()

set(CPACK_PACKAGE_DESCRIPTION "Qt for MCU")
set(CPACK_PACKAGE_VERSION "${CMAKE_PROJECT_VERSION}")
set(CPACK_PACKAGE_VENDOR "The Qt Company")
set(CPACK_PACKAGE_CONTACT "https://www.qt.io/contact-us")
set(CPACK_PACKAGE_NAME "Qul_${CMAKE_PROJECT_VERSION}")
set(CPACK_PACKAGE_FILE_NAME ${CPACK_PACKAGE_NAME})
set(CPACK_GENERATOR TZST)
set(CPACK_THREADS 0) # When set to 0 it will use all available cores to do packing
set(CPACK_ARCHIVE_CORELIBRARY_FILE_NAME "qtformcus-core-library-${CORE_PACKAGE_NAME}")
set(CPACK_ARCHIVE_COREHEADERS_FILE_NAME "qtformcus-core-headers-${CORE_PACKAGE_NAME}")
set(CPACK_ARCHIVE_TOOLS_FILE_NAME "qtformcus-tools-${CORE_PACKAGE_NAME}")
set(CPACK_ARCHIVE_COREBASE_FILE_NAME "qtformcus-core-base")
set(CPACK_ARCHIVE_DOCS_FILE_NAME "qtformcus-documentation")
set(CPACK_ARCHIVE_EXAMPLES_FILE_NAME "qtformcus-examples")
set(CPACK_ARCHIVE_CORESOURCES_FILE_NAME "qtformcus-core-sources-${CMAKE_PROJECT_VERSION}")
set(CPACK_ARCHIVE_PLATFORMSOURCES_FILE_NAME "qtformcus-platform-sources-${QUL_PLATFORM}-${QUL_COMPILER_NAME}-${CMAKE_PROJECT_VERSION}")
set(CPACK_ARCHIVE_PLATFORMHEADERS_FILE_NAME "qtformcus-platform-headers-${CMAKE_PROJECT_VERSION}") # Generic headers not specific to any platform or compiler
set(CPACK_ARCHIVE_PLATFORMBASE_FILE_NAME "qtformcus-platform-base-${CMAKE_PROJECT_VERSION}") # Platform base package must not contain any platform or architecture specific files
set(CPACK_ARCHIVE_PLATFORMLIBRARY_FILE_NAME "qtformcus-platform-library-${QUL_PLATFORM}-${QUL_COMPILER_NAME}-${CMAKE_PROJECT_VERSION}")
set(CPACK_ARCHIVE_TESTAUTOMATION_FILE_NAME "test-automation-${QUL_PLATFORM}-${QUL_COMPILER_NAME}-${CMAKE_PROJECT_VERSION}")
set(CPACK_ARCHIVE_TESTSSOURCES_FILE_NAME "qtformcus-tests-sources-${CMAKE_PROJECT_VERSION}")
set(CPACK_ARCHIVE_HARDWARECLOUD_FILE_NAME "hardwarecloud-${CORE_PACKAGE_NAME}")
set(CPACK_ARCHIVE_TESTBINS_FILE_NAME "qtformcus-testbins-${QUL_PLATFORM}-${QUL_COMPILER_NAME}-${CMAKE_PROJECT_VERSION}")

# TestBins in desktop debug build are not used so that it would be excluded. Also disable them while building host tools.
if (CMAKE_CROSSCOMPILING OR (NOT ${CMAKE_BUILD_TYPE} STREQUAL "Debug" AND HOST_TOOLS_EXIST))
    set(CPACK_COMPONENTS_ALL "${CPACK_COMPONENTS_ALL};TestBins")
endif()

if(QUL_BUILD_FRAMEWORK)
    set(CPACK_COMPONENTS_ALL "${CPACK_COMPONENTS_ALL};CoreLibrary;CoreSources;CoreBase;CoreHeaders;TestsSources")
    set(CPACK_COMPONENTS_ALL "${CPACK_COMPONENTS_ALL};Examples;PlatformBase;PlatformHeaders")
endif()

if(QUL_BUILD_TOOLS)
    set(CPACK_COMPONENTS_ALL "${CPACK_COMPONENTS_ALL};Tools")
endif()

if(QUL_BUILD_DOC)
    set(CPACK_COMPONENTS_ALL "${CPACK_COMPONENTS_ALL};Docs")
endif()

if(QUL_BUILD_PLATFORM)
    set(CPACK_COMPONENTS_ALL "${CPACK_COMPONENTS_ALL};PlatformSources;PlatformLibrary")
endif()

if(CMAKE_CROSSCOMPILING)
    set(CPACK_COMPONENTS_ALL "${CPACK_COMPONENTS_ALL};TestAutomation")
endif()

if(QUL_BUILD_HARDWARECLOUD)
  set(CPACK_COMPONENTS_ALL "${CPACK_COMPONENTS_ALL};HardwareCloud")
endif()

set(CPACK_ARCHIVE_COMPONENT_INSTALL ON)
set(CPACK_COMPONENTS_GROUPING IGNORE)
set(CPACK_COMPONENT_INCLUDE_TOPLEVEL_DIRECTORY ON)
set(CPACK_TOPLEVEL_TAG "Qul_${CMAKE_PROJECT_VERSION}")

include (CPack)
